/*
 *
 *  Replication module.
 *	(c) IBPhoenix, 2014-2015
 *
 */

#ifndef FIREBIRD_REPL_IFACE_H
#define FIREBIRD_REPL_IFACE_H

#include "firebird/Interface.h"

namespace Firebird
{
class IField
{
public:
	virtual bool hasData() = 0;
	virtual const char* getName() = 0;
	virtual int getType() = 0;
	virtual int getScale() = 0;
	virtual int getLength() = 0;
	virtual int getCharset() = 0;
	virtual bool isNull() = 0;
	virtual void* getData() = 0;
	virtual bool next() = 0;
};

class IRecord
{
public:
	virtual const char* getSchemaName() = 0;
	virtual const char* getTableName() = 0;
	virtual long long getNumber() = 0;
	virtual IField* getField() = 0;
};

class IReplTransaction
{
public:
	virtual void dispose() = 0;
	virtual intptr_t processInsert(IStatus* status, IAttachment* att, ITransaction* tra, IRecord* new_data) = 0;
	virtual intptr_t processUpdate(IStatus* status, IAttachment* att, ITransaction* tra, IRecord* old_data, IRecord* new_data) = 0;
	virtual intptr_t processDelete(IStatus* status, IAttachment* att, ITransaction* tra, IRecord* old_data) = 0;
	virtual intptr_t processSequence(IStatus* status, const char* name, long long value) = 0;
	virtual intptr_t finishReplication(IStatus* status) = 0;
	virtual void startSavepoint(IStatus* status, long number) = 0;
	virtual void releaseSavepoint(IStatus* status, long number) = 0;
	virtual void rollbackSavepoint(long number) = 0;
};

class IReplPlugin
{
public:
	virtual void dispose() = 0;
	virtual IReplTransaction* initReplication(IStatus* status, long long tra_number) = 0;
};

#define REPL_PLUGIN_ENTRYPOINT_NAME "getReplPlugin"
typedef IReplPlugin* ReplPluginEntrypoint(IStatus* status, int Version, IConfig* config);

} // namespace

#endif
